@@ -1,23 +1,22 @@
-#
 # Copyright 2013 OpenStack LLC
-# All Rights Reserved
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
+# Licensed under the Apache License, Version 2.0 (the "License"); you may not
+# use this file except in compliance with the License. You may obtain a copy of
+# the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
-import time
+# License for the specific language governing permissions and limitations under
+# the License.
 
-import testtools
+import time
+import uuid
 
 from pycadf import attachment
+from pycadf import cadftype
 from pycadf import credential
 from pycadf import endpoint
 from pycadf import event
@@ -30,10 +29,11 @@ from pycadf import reason
 from pycadf import reporterstep
 from pycadf import resource
 from pycadf import tag
+from pycadf.tests import base
 from pycadf import timestamp
 
 
-class TestCADFSpec(testtools.TestCase):
+class TestCADFSpec(base.TestCase):
     def test_endpoint(self):
         endp = endpoint.Endpoint(url='http://192.168.0.1',
                                  name='endpoint name',
@@ -61,6 +61,21 @@ class TestCADFSpec(testtools.TestCase):
         for key in credential.CRED_KEYNAMES:
             self.assertIn(key, dict_cred)
 
+    def test_federated_credential(self):
+        cred = credential.FederatedCredential(
+            token=identifier.generate_uuid(),
+            type='http://docs.oasis-open.org/security/saml/v2.0',
+            identity_provider=identifier.generate_uuid(),
+            user=identifier.generate_uuid(),
+            groups=[
+                identifier.generate_uuid(),
+                identifier.generate_uuid(),
+                identifier.generate_uuid()])
+        self.assertEqual(cred.is_valid(), True)
+        dict_cred = cred.as_dict()
+        for key in credential.FED_CRED_KEYNAMES:
+            self.assertIn(key, dict_cred)
+
     def test_geolocation(self):
         geo = geolocation.Geolocation(id=identifier.generate_uuid(),
                                       latitude='43.6481 N',
@@ -283,3 +298,24 @@ class TestCADFSpec(testtools.TestCase):
                               target=resource.Resource(typeURI='storage'),
                               observer=resource.Resource(id='target'),
                               outcome='success'))
+
+    def _create_none_validator_descriptor(self):
+        class Owner(object):
+            x = cadftype.ValidatorDescriptor(uuid.uuid4().hex)
+
+        owner = Owner()
+        owner.x = None
+
+    def test_invalid_value_descriptor(self):
+        """Test setting a ValidatorDescriptor to None results in ValueError"""
+
+        self.assertRaises(ValueError, self._create_none_validator_descriptor)
+
+    def test_cadfabstracttype_attribute_error(self):
+        """Test an invalid CADFAbstractType attribute is set returns False"""
+
+        h = host.Host(id=identifier.generate_uuid(),
+                      address='192.168.0.1',
+                      agent='client',
+                      platform='AIX')
+        self.assertEqual(h._isset(uuid.uuid4().hex), False)
